<?php
// This file is part of Moodle - http://moodle.org/.
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot . '/blocks/infoscours/edit_form.php');
require_once($CFG->dirroot . '/blocks/infoscours/classes/form/infoscours_partial_edit_form.php');

/**
 * Class block_infoscours
 *
 * Main file for the infoscours block.
 *
 * @package    block_infoscours
 * @copyright  UNESS | Pimenko 2024
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class block_infoscours extends block_base {

    /**
     * Initializes the object by setting the title property to the value obtained from the get_string function.
     *
     * @return void
     */
    public function init(): void {
        $this->title = get_string(
            'pluginname',
            'block_infoscours',
        );
    }

    /**
     * Checks if the application has a configuration setup.
     *
     * @return bool Returns true if the application has a configuration setup, false otherwise.
     */
    public function has_config(): bool {
        return true;
    }

    /**
     * Returns the applicable formats for the given method.
     *
     * @return array Returns an associative array of applicable formats.
     *     The key represents the format, and the value represents whether the format is applicable or not.
     */
    public function applicable_formats(): array {
        return [ 'course-view' => true ];
    }

    /**
     * Retrieves the content of the block.
     *
     * @return stdClass|null
     * Returns the content of the block as an instance of stdClass if the application has a configuration
     * setup, or an empty string if the user is not in editing mode.
     */
    public function get_content() {
        global $COURSE;

        $context = context_course::instance($this->page->course->id);

        if (empty($this->config)) {
            $this->config = new stdClass();
        }

        // Check if content is already set or configuration is null.
        if (!$this->page->user_is_editing() || !has_capability(
                'block/infoscours:addinstance',
                $context,
            )) {
            return $this->content;
        }

        // Initialize content object.
        $this->content = new stdClass;
        $this->content->footer = '';

        $this->config->pluginname = $this->instance->blockname;
        $this->config->blockid = $this->instance->id;

        // Edit config icon - always show - needed for positioning UI.
        $str = get_string(
            'seeform',
            'block_infoscours',
        );
        $actionurl = $this->page->url->out(
            false,
            [ 'sesskey' => sesskey() ],
        );
        $editactionurl = new moodle_url(
            $actionurl,
            [ 'bui_editid' => $this->config->blockid ],
        );
        $editactionurl->remove_params([ 'sesskey' ]);

        $link = new action_menu_link_secondary(
            $editactionurl,
            null,
            $str,
            [
                'class' => 'editing_edit btn btn-primary m-auto d-block',
                'data-action' => 'editblock',
                'data-blockid' => $this->config->blockid,
                'data-blockform' => 'block_infoscours_edit_form',
                'data-header' => $str,
            ],
        );

        $renderer = $this->page->get_renderer('core');
        $formbutton = $renderer->render_from_template(
            'core/action_menu_link',
            $link->export_for_template($renderer),
        );

        $this->content->text =
            '<div class="alert alert-primary" role="alert">' .
            get_string('blockcontentintro', 'block_infoscours') .
            '</div>';

        $mform = new infoscours_partial_edit_form($this->config);
        $mform->set_data($this->config);

        if ($data = $mform->get_data()) {
            // In case the form is submitted and validated, process the data.
            $blockbase = $this->page->blocks->find_instance($this->instance->id);

            $mform->save_data(
                $data,
                $blockbase,
            );
            redirect(
                $editactionurl
            );
        } else {
            // Either form hasn't been submitted or data didn't validate.
            // In either case, display the form.
            $this->content->text .= $mform->render();
        }

        $authorcount = 0;
        for ($i = 1; $i <= 3; $i++) {
            if (!empty($this->config->{'additional' . $i . '_lastname'}) ||
                !empty($this->config->{'additional' . $i . '_firstname'}) ||
                !empty($this->config->{'additional' . $i . '_email'})) {
                $authorcount++;
            }
        }

        $this->page->requires->js_call_amd(
            'block_infoscours/addauthor',
            'init',
            [ 'authorcount' => $authorcount ],
        );

        if (isset($this->config) && (
                !isset($this->config->main_firstname) || $this->config->main_firstname === ''
                || !isset($this->config->main_lastname) || $this->config->main_lastname === ''
                || !isset($this->config->main_email) || $this->config->main_email === ''
                || !isset($this->config->license) || $this->config->license === ''
            )
        ) {
            $this->page->requires->js_call_amd(
                'block_infoscours/checkrequiredfield',
                'init',
                [
                    'show' => true,
                    'formbutton' => $formbutton
                ],
            );
        }

        return $this->content;
    }

    /**
     * Determines whether multiple instances of the method's class are allowed.
     *
     * @return bool Returns false to indicate that only one instance of the class is allowed.
     */
    public function instance_allow_multiple(): bool {
        // Returns false to allow only one instance per course.
        return false;
    }
}
