<?php
// This file is part of Moodle - http://moodle.org/.
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

defined('MOODLE_INTERNAL') || die();

require_once($CFG->libdir . '/formslib.php');
require_once(__DIR__ . '/../../lib.php');
require_once($CFG->dirroot . '/course/lib.php');
require_once($CFG->dirroot . '/mod/label/lib.php');

use core_courseformat\stateactions;
use core_courseformat\stateactions as stateactions_base;

/**
 * Class infoscours_partial_edit_form
 *
 * This class represents a form for editing partial information in the block instance.
 *
 * @package    block_infoscours
 * @copyright  UNESS | Pimenko 2024
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 *
 */
class infoscours_partial_edit_form extends moodleform {

    /**
     * Represents the configuration settings for the block.
     *
     * @var object
     */
    private object $config;

    /**
     * __construct method.
     *
     * Initializes a new instance of the class.
     *
     * @param mixed $config The configuration for the class.
     * @return void
     */
    public function __construct($config) {
        $this->config = $config;
        parent::__construct();
    }

    /**
     * Defines the form elements for the block instance.
     *
     * @return void
     */
    protected function definition() {
        global $COURSE, $CFG, $PAGE;

        $mform = $this->_form;

        $mform->addElement(
            'hidden',
            'id',
            $COURSE->id,
        );
        $mform->setType(
            'id',
            PARAM_INT,
        );

        $mform->addElement(
            'text',
            'config_main_lastname',
            get_string(
                'lastname',
                'block_infoscours',
            ),
        );
        $mform->setType(
            'config_main_lastname',
            PARAM_NOTAGS,
        );
        $mform->addRule(
            'config_main_lastname',
            get_string('required'),
            'required',
            null,
            'client',
        );

        $mform->addElement(
            'text',
            'config_main_firstname',
            get_string(
                'firstname',
                'block_infoscours',
            ),
        );
        $mform->setType(
            'config_main_firstname',
            PARAM_NOTAGS,
        );
        $mform->addRule(
            'config_main_firstname',
            get_string('required'),
            'required',
            null,
            'client',
        );

        $mform->addElement(
            'text',
            'config_main_email',
            get_string(
                'email',
                'block_infoscours',
            ),
        );
        $mform->setType(
            'config_main_email',
            PARAM_EMAIL,
        );
        $mform->addRule(
            'config_main_email',
            get_string('required'),
            'required',
            null,
            'client',
        );

        // Generate license field.
        $licenseconfig = $CFG->block_infoscours_licenses;
        $categoriesconfig = $CFG->block_infoscours_categories;
        $currentcategoryid = $PAGE->course->category;
        $matchinglineitems = getmatchinglineitems(
            $categoriesconfig,
            $currentcategoryid,
            true
        );

        $licenseslines = explode(
            "\n",
            $licenseconfig,
        );
        $licenses = [];
        foreach ($licenseslines as $licenseline) {
            $parts = explode(
                '|',
                $licenseline,
            );
            if (count($parts) >= 2) {
                $id = trim($parts[0]);
                $name = trim($parts[1]);
                if (!key_exists(
                    $id,
                    $licenses,
                )) {
                    $licenses[$id] = $name;
                }
            }
        }
        $mform->addElement(
            'select',
            'config_license',
            get_string(
                'license',
                'block_infoscours',
            ),
            $licenses,
        );
        $mform->addRule(
            'config_license',
            get_string('required'),
            'required',
            null,
            'client',
        );

        if ($this->config->license) {
            $mform->setDefault(
                'config_license',
                $this->config->license,
            );
        } else {
            $mform->setDefault(
                'config_license',
                intval(end($matchinglineitems)),
            );
        }

        $this->add_action_buttons(false, get_string('seeform', 'block_infoscours'));
    }

    /**
     * Saves the data for the block instance.
     *
     * @param object $data The data to be saved.
     * @param block_base $blockinstance The block instance.
     * @param object $config The block configuration.
     *
     * @return void
     */
    public function save_data($data, block_base $blockinstance) {
        global $COURSE;

        $config = $this->config;

        // Modify the configuration parameters as per your requirements.
        $config->main_firstname = $data->config_main_firstname;
        $config->main_lastname = $data->config_main_lastname;
        $config->main_email = $data->config_main_email;
        $config->license = $data->config_license;
        if (isset($data->config_moduleinfotopid)) {
            $config->moduleinfotopid = $data->config_moduleinfotopid;
        }
        if (isset($data->config_moduleinfobottomid)) {
            $config->moduleinfobottomid = $data->config_moduleinfobottomid;
        }
        $config->infoscourstags = \core_tag_tag::get_item_tags_array('core', 'course', $COURSE->id);
        $config->infoscourstags = array_values($config->infoscourstags);

        // Create label activities.
        $config = createlabelmod($config);

        // Use instance_config_save to save the configuration changes.
        $blockinstance->instance_config_save($config);
    }

    /**
     * Validates data and returns any errors found.
     *
     * @param array $data The data to be validated.
     * @param array $files The files to be validated.
     * @return array An array of errors, if any.
     */
    public function validation($data, $files): array {
        $errors = parent::validation(
            $data,
            $files,
        );

        if (empty($data['config_main_email'])) {
            // Check if email is valid.
            if (!filter_var(
                $data['config_main_email'],
                FILTER_VALIDATE_EMAIL,
            )) {
                $errors['config_main_email'] = get_string('invalidemail');
            }
        }

        return $errors;
    }

    /**
     * Sets the data for the block instance.
     *
     * @param mixed $defaults The default values for the block instance.
     *
     * @return void
     */
    public function set_data($defaults) {
        // Check if the block instance exists.
        if ($defaults) {
            // Get the block instance config.
            $defaults = get_object_vars($defaults);

            // Specify the keys you are interested in.
            $keys = [
                'main_firstname',
                'main_lastname',
                'main_email',
                'license',
            ];

            foreach ($keys as $key) {
                if (isset($defaults[$key])) {
                    $element = $this->_form->getElement('config_' . $key);
                    if ($element !== null) {
                        $element->setValue($defaults[$key]);
                    }
                }
            }
        }

        parent::set_data($defaults);
    }
}
