<?php
// This file is part of Moodle - http://moodle.org/.
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Settings for the infoscours block.
 *
 * @package    block_infoscours
 * @copyright  UNESS | Pimenko 2024
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class admin_setting_license extends admin_setting_configtextarea {

    /**
     * Validates the given data.
     * Removes unnecessary whitespace from the data.
     * Splits the data into lines.
     * Configures a pattern to check each line.
     * Adds a line index.
     * Iterates through each line, ignoring empty lines.
     * Validates each line using the pattern.
     * If a line is invalid, returns an error message with the line number.
     * Increments the line index.
     * If all lines are valid, returns true.
     *
     * @param string $data The data to validate.
     * @return bool|string True if the data is valid, or an error message with the line number if invalid.
     */
    public function validate($data) {
        // Remove unnecessary whitespace.
        $data = trim($data);

        // Split into lines.
        $lines = explode(
            "\n",
            $data,
        );

        // Define pattern to validate each line.
        // Leave this in single line.
        $pattern = "/^[0-9]+\|[^|]+\|(https?:\/\/[^|]+)?\|(https?:\/\/[^|]+)?$/";

        // Add line index.
        $lineindex = 1;

        foreach ($lines as $line) {
            // Ignore empty lines.
            if (empty($line)) {
                continue;
            }

            // Remove potential carriage return at line end due to Windows-style line endings (\r\n).
            $line = str_replace(
                "\r",
                "",
                $line,
            );

            // Validate each line with the pattern.
            if (!preg_match(
                $pattern,
                $line,
            )) {
                // Return an error message with the line number.
                return get_string(
                        'invalidlicense',
                        'block_infoscours',
                    ) . " Line: " . $lineindex;
            }
            // Increment line index.
            $lineindex++;
        }

        // If all lines are valid, return true.
        return true;
    }
}
