<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot . '/blocks/edit_form.php');
require_once(__DIR__ . '/lib.php');

/**
 * Class block_infoscours_edit_form
 *
 * Form for editing block settings.
 *
 * @package    block_infoscours
 * @copyright  UNESS | Pimenko 2024
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class block_infoscours_edit_form extends block_edit_form {

    /**
     * The variable $authorcount represents the count of authors.
     *
     * @var int $authorcount
     */
    private int $authorcount = 0;

    /**
     * Add an element of the specified type to a MoodleQuickForm instance.
     *
     * @param MoodleQuickForm $mform The MoodleQuickForm instance to which the element will be added.
     * @param string $type The type of the element to add.
     * @param string $name The name of the element.
     * @param mixed $label The label for the element.
     * @param mixed|null $attributes Additional attributes for the element (optional).
     * @param string $paramtype The type of the parameter (optional).
     * @return void
     */
    private function add_element_with_type($mform, string $type, string $name, $label, $attributes = null,
        string $paramtype = ''): void {
        $mform->addElement(
            $type,
            'config_' . $name,
            get_string(
                $label,
                'block_infoscours',
            ),
            $attributes,
        );
        $mform->setType(
            'config_' . $name,
            $paramtype,
        );
    }

    /**
     * Add a textarea element to a MoodleQuickForm instance.
     *
     * @param MoodleQuickForm $mform The MoodleQuickForm instance to which the element will be added.
     * @param string $name The name of the textarea element.
     * @param mixed|null $attributes The attributes for the textarea element. Defaults to null.
     * @return void
     */
    private function add_htmleditor_element(MoodleQuickForm $mform, string $name, $attributes = null): void {
        $this->add_element_with_type(
            $mform,
            'editor',
            $name,
            $name,
            $attributes,
            PARAM_RAW,
        );
    }

    /**
     * Add a text element to a MoodleQuickForm instance.
     *
     * @param MoodleQuickForm $mform The MoodleQuickForm instance to which the element will be added.
     * @param string $name The name of the text element.
     * @param string $paramtype The type of the text element (e.g. 'email', 'password', etc.).
     * @param mixed $attributes (optional) Additional attributes for the text element.
     * @return void
     */
    private function add_text_element(MoodleQuickForm $mform, string $name, string $paramtype, $attributes = null): void {
        $this->add_element_with_type(
            $mform,
            'text',
            $name,
            $name,
            $attributes,
            $paramtype,
        );
    }

    /**
     * Add a select element to a MoodleQuickForm instance.
     *
     * @param MoodleQuickForm $mform The MoodleQuickForm instance to which the element will be added.
     * @param string $name The name of the select element.
     * @param array $options The options for the select element.
     * @return void
     */
    private function add_select_element(MoodleQuickForm $mform, string $name, array $options) {
        // Add empty value.
        $mform->addElement(
            'select',
            'config_' . $name,
            get_string(
                $name,
                'block_infoscours',
            ),
            $options,
        );
    }

    /**
     * The definition method.
     *
     * @return void
     */
    public function definition(): void {
        global $CFG, $COURSE, $OUTPUT, $PAGE;

        $mform =& $this->_form;
        $categoriesconfig = $CFG->block_infoscours_categories;
        $currentcategoryid = $this->page->course->category;
        $matchinglineitems = getmatchinglineitems(
            $categoriesconfig,
            $currentcategoryid,
            true,
        );

        $mform->addElement(
            'hidden',
            'blockid',
            $this->block->instance->id,
        );
        $mform->setType(
            'blockid',
            PARAM_INT,
        );
        $mform->addElement(
            'hidden',
            'blockname',
            $this->optional_param(
                'blockname',
                null,
                PARAM_PLUGIN,
            ),
        );
        $mform->setType(
            'blockname',
            PARAM_PLUGIN,
        );
        $mform->addElement(
            'hidden',
            'blockregion',
            $this->optional_param(
                'blockregion',
                null,
                PARAM_TEXT,
            ),
        );
        $mform->setType(
            'blockregion',
            PARAM_TEXT,
        );
        $mform->addElement(
            'hidden',
            'pagehash',
            $this->optional_param(
                'pagehash',
                null,
                PARAM_ALPHANUMEXT,
            ),
        );
        $mform->setType(
            'pagehash',
            PARAM_ALPHANUMEXT,
        );

        // Def. bui_defaultregion. ==> 'side-pre'.
        $mform->addElement(
            'hidden',
            'bui_defaultregion',
            'side-pre',
        );
        $mform->setType(
            'bui_defaultregion',
            PARAM_TEXT,
        );

        // Def. bui_defaultweight. ==> value '0'.
        $mform->addElement(
            'hidden',
            'bui_defaultweight',
            '0',
        );
        $mform->setType(
            'bui_defaultweight',
            PARAM_TEXT,
        );

        // Def. bui_visible. ==> value '1'.
        $mform->addElement(
            'hidden',
            'bui_visible',
            '1',
        );
        $mform->setType(
            'bui_visible',
            PARAM_TEXT,
        );

        // Def. bui_region. ==> value 'side-pre'.
        $mform->addElement(
            'hidden',
            'bui_region',
            'side-pre',
        );
        $mform->setType(
            'bui_region',
            PARAM_TEXT,
        );
        // Def. weight visible. ==> value '1'.
        $mform->addElement(
            'hidden',
            'bui_visible',
            '1',
        );
        $mform->setType(
            'bui_visible',
            PARAM_TEXT,
        );
        // Def. weight visible. ==> value '0'.
        $mform->addElement(
            'hidden',
            'bui_weight',
            '0',
        );
        $mform->setType(
            'bui_weight',
            PARAM_TEXT,
        );

        // mod generated ids if present.
        $mform->addElement(
            'hidden',
            'config_moduleinfotopid',
        );
        $mform->setType(
            'moduleinfotopid',
            PARAM_INT,
        );

        $mform->addElement(
            'hidden',
            'config_moduleinfobottomid',
        );
        $mform->setType(
            'moduleinfobottomid',
            PARAM_INT,
        );

        $this->add_header(
            $mform,
            'editablesettings',
            'config_editableheader',
        );

        $this->add_authorfields(
            $mform,
            'main',
            get_string(
                'mainauthor',
                'block_infoscours',
            ),
            true,
        );
        for ($i = 1; $i <= 3; $i++) {
            $this->add_authorfields(
                $mform,
                "additional$i",
                get_string(
                    "additionalauthor$i",
                    'block_infoscours',
                ),
                false,
            );
        }

        if(!$this->block->config->authorteam['text']) {
            $attributes = [
                'class' => 'authorteam d-none'
            ];
        } else {
            $attributes = [
                'class' => 'authorteam',
            ];
        }
        $this->add_htmleditor_element(
            $mform,
            'authorteam',
            $attributes
        );

        $buttonarray = [];

        $addauthorparam = [
            'data-action' => 'block_infoscours/add_author_button',
        ];

        if ($this->authorcount === 3) {
            $addauthorparam['disabled'] = true;
        }

        $buttonarray[] = $mform->createElement(
            'button',
            'add_author_button',
            get_string(
                'addauthor',
                'block_infoscours',
            ),
            $addauthorparam,
        );

        $attributes = [
            'data-action' => 'block_infoscours/addteam_button',
        ];

        if ($this->block->config->authorteam['text']) {
            $attributes['disabled'] = true;
        }

        $buttonarray[] = $mform->createElement(
            'button',
            'add_team_button',
            get_string(
                'addteam',
                'block_infoscours',
            ),
            $attributes,
        );
        $mform->addGroup(
            $buttonarray,
            'buttonar',
            '',
            [ ' ' ],
            false,
        );

        // Generate license field.
        $licenseconfig = $CFG->block_infoscours_licenses;
        $licenseslines = explode(
            "\n",
            $licenseconfig,
        );
        $licenses = [];
        foreach ($licenseslines as $licenseline) {
            $parts = explode(
                '|',
                $licenseline,
            );
            if (count($parts) >= 2) {
                $id = trim($parts[0]);
                $name = trim($parts[1]);
                if (!key_exists(
                    $id,
                    $licenses,
                )) {
                    $licenses[$id] = $name;
                }
            }
        }
        $mform->addElement(
            'select',
            'config_license',
            get_string(
                'license',
                'block_infoscours',
            ),
            $licenses,
        );
        $mform->addRule(
            'config_license',
            get_string('required'),
            'required',
            null,
            'client',
        );
        $mform->setDefault(
            'config_license',
            intval(end($matchinglineitems)),
        );

        $this->add_text_element(
            $mform,
            'contact_email',
            PARAM_TEXT,
            [
                'placeholder' => get_string(
                    'placeholder_contactemail',
                    'block_infoscours',
                )
            ],
        );

        // Discipline - Dropdown List.
        $disciplinesoptions = getdisciplinesoptions();
        $this->add_select_element(
            $mform,
            'discipline',
            $disciplinesoptions,
        );

        // Cycle - Dropdown List.
        $cyclesoptions = getcyclesoptions();
        $this->add_select_element(
            $mform,
            'cycle',
            $cyclesoptions,
        );

        // Level / Year - Dropdown List.
        $niveauxoptions = getniveauxoptions();
        $this->add_select_element(
            $mform,
            'niveau',
            $niveauxoptions,
        );

        $this->add_htmleditor_element(
            $mform,
            'objectives',
        );
        $this->add_htmleditor_element(
            $mform,
            'prerequis',
        );
        $this->add_htmleditor_element(
            $mform,
            'presentation_cours',
        );
        $this->add_text_element(
            $mform,
            'duree_heures',
            PARAM_TEXT,
        );
        $this->add_htmleditor_element(
            $mform,
            'modes_evaluation',
        );

        $this->add_text_element(
            $mform,
            'credits_ects',
            PARAM_TEXT,
        );
        $this->add_htmleditor_element(
            $mform,
            'bibliography',
        );

        $coursetags = core_tag_tag::get_item_tags_array(
            'core',
            'course',
            $COURSE->id,
        );
        // Prevent to load old plugin settings.
        if ($this->block->config && $this->block->config->infoscourstags) {
            $this->block->config->infoscourstags = $coursetags;
        }
        $mform->addElement(
            'tags',
            'config_infoscourstags',
            get_string('tags'),
        );
        $mform->setDefault(
            'config_infoscourstags',
            $coursetags,
        );

        $mform->addElement(
            'advcheckbox',
            'config_check_ai',
            get_string(
                'ai_usage_consent',
                'block_infoscours',
            ),
            get_string(
                'ai_usage_consent_subtext',
                'block_infoscours',
            ),
            null,
            [
                0,
                1,
            ],
        );
        $mform->setDefault(
            'config_check_ai',
            0,
        );

        // Prepare the context for Mustache.
        $context = (object) [
            'modaltitle' => get_string(
                'modaltitle',
                'block_infoscours',
            ),
            'modalbody' => get_string(
                'modalbody',
                'block_infoscours',
            ),
            'refuse' => get_string(
                'refuse',
                'block_infoscours',
            ),
            'authorize' => get_string(
                'authorize',
                'block_infoscours',
            ),
        ];

        // Render the template.
        $modalhtml = $OUTPUT->render_from_template(
            'block_infoscours/ai_consent_modal',
            $context,
        );

        // Add the modal HTML to the form.
        $mform->addElement(
            'static',
            'modal',
            '',
            $modalhtml,
        );

        // We should use $this->page but for some reasons JS is not load if we do ...
        $PAGE->requires->js_call_amd(
            'block_infoscours/addauthor',
            'init',
            [ 'authorcount' => $this->authorcount ],
        );

        $this->add_header(
            $mform,
            'generalsettings',
            'config_generalheader',
        );

        if (!empty($matchinglineitems)) {
            $this->add_category_info(
                $mform,
                $matchinglineitems,
            );
        } else {
            $mform->addElement(
                'html',
                get_string(
                    'generalsettingsempty',
                    'block_infoscours',
                ),
            );
        }

        if (empty($this->_customdata) || is_null($this->_customdata['actionbuttons'])) {
            $this->add_action_buttons();
        }
    }

    /**
     * Adds category information to the form fields.
     *
     * @param MoodleQuickForm $mform The form instance.
     * @param array $matchinglineitems The matching line items obtained from the configuration.
     * @return void
     */
    private function add_category_info($mform, array $matchinglineitems): void {
        $fields = [
            'projectpresentation',
            'logo1url',
            'logo1alt',
            'logo2url',
            'logo2alt',
            'logo3url',
            'logo3alt',
            'logo4url',
            'logo4alt',
        ];
        for ($i = 0; $i < count($fields); $i++) {
            if (strpos(
                    $fields[$i],
                    'url',
                ) !== false) {
                // Check if the field is a URL.
                $logourl = $matchinglineitems[$i + 1] ?? '';
                if ($logourl) {
                    // Create image element.
                    $label = get_string(
                        $fields[$i],
                        'block_infoscours',
                    );
                    $contenthtml = "<div class='form-group row fitem'>
                        <div class='col-md-3 col-form-label d-flex pb-0 pr-md-0'><label>{$label}</label></div>
                        <div class='col-md-9 form-inline align-items-start felement'>
                        <img class='blockinfoimg' src='{$logourl}' alt='{$fields[$i]}' /></div>
                    </div>";

                    $mform->addElement(
                        'html',
                        $contenthtml,
                    );
                }
            } else {
                $this->add_disabled_textarea(
                    $mform,
                    $fields[$i],
                    $matchinglineitems[$i + 1] ?? '',
                );
            }
        }
    }

    /**
     * Add a header element to the configuration header.
     *
     * @param MoodleQuickForm $mform The header text.
     * @param string $header The header text.
     * @param string $configheader The configuration header.
     * @return void
     */
    private function add_header($mform, string $header, string $configheader): void {
        $mform->addElement(
            'header',
            $configheader,
            get_string(
                $header,
                'block_infoscours',
            ),
        );
    }

    /**
     * Adds a disabled textarea element to the given Moodle form.
     *
     * @param MoodleQuickForm $mform The Moodle form to add the element to.
     * @param string $name The name attribute of the textarea element.
     * @param mixed $value The default value of the textarea element.
     *
     * @return void
     */
    private function add_disabled_textarea($mform, $name, $value): void {
        $mform->addElement(
            'textarea',
            $name,
            get_string(
                $name,
                'block_infoscours',
            ),
            [
                'disabled' => 'disabled',
            ],
        );
        $mform->getElement($name)->setValue($value);
        $mform->setType(
            $name,
            PARAM_TEXT,
        );
    }

    /**
     * Adds author fields to the given Moodle form.
     *
     * @param object $mform The Moodle form object to add the fields to.
     * @param string $prefix The prefix to use for the field names.
     * @param string $groupname The name of the field group.
     * @param bool $required True if the fields should be required, false otherwise.
     *
     * @return void
     */
    protected function add_authorfields($mform, $prefix, $groupname, $required) {
        global $CFG;

        // Generate license field.
        $schoollist = $CFG->block_infoscours_schoollist;
        $schoollines = explode(
            "\n",
            $schoollist,
        );
        $schools = [];
        $schools[0] = get_string('none', 'block_infoscours');
        foreach ($schoollines as $schoolline) {
            $parts = explode(
                '|',
                $schoolline,
            );
            if (count($parts) >= 2) {
                $id = trim($parts[0]);
                $name = trim($parts[1]);
                if (!key_exists(
                    $id,
                    $schools,
                )) {
                    $schools[$id] = $name;
                }
            }
        }

        $authorfields = [
            $mform->createElement(
                'text',
                'config_' . $prefix . '_lastname',
                '',
                [
                    'placeholder' => get_string(
                        'lastname',
                        'block_infoscours',
                    ),
                ],
            ),
            $mform->createElement(
                'text',
                'config_' . $prefix . '_firstname',
                '',
                [
                    'placeholder' => get_string(
                        'firstname',
                        'block_infoscours',
                    ),
                ],
            ),
            $mform->createElement(
                'text',
                'config_' . $prefix . '_email',
                '',
                [
                    'placeholder' => get_string(
                        'email',
                        'block_infoscours',
                    ),
                ],
            ),
            $mform->createElement(
                'text',
                'config_' . $prefix . '_fonction',
                '',
                [
                    'placeholder' => get_string(
                        'fonction',
                        'block_infoscours',
                    ),
                ],
            ),
            $mform->createElement(
                'select',
                'config_' . $prefix . '_school',
                '',
                $schools,
            ),
        ];

        if ($required) {
            $mform->addGroup(
                $authorfields,
                $prefix . '_group',
                $groupname,
                [
                    ' ',
                    ' ',
                    ' ',
                ],
                false,
            );
            $mform->addRule(
                $prefix . '_group',
                get_string('required'),
                'required',
                null,
                'client',
            );
        } else {
            if ($this->block->config &&
                (!$this->block->config->{$prefix . '_lastname'} ||
                    !$this->block->config->{$prefix . '_firstname'} ||
                    !$this->block->config->{$prefix . '_email'})) {
                $mform->addGroup(
                    $authorfields,
                    $prefix . '_group',
                    $groupname,
                    [
                        ' ',
                        ' ',
                        ' ',
                    ],
                    false,
                    [ 'class' => 'd-none authorfield' ],
                );
            } else {
                $mform->addGroup(
                    $authorfields,
                    $prefix . '_group',
                    $groupname,
                    [
                        ' ',
                        ' ',
                        ' ',
                    ],
                    false,
                    [ 'class' => 'authorfield authorshow' ],
                );

                // We need this to know how many element are displayed.
                if ($this->block->config) {
                    $this->authorcount++;
                }
            }
        }

        $mform->setType(
            'config_' . $prefix . '_lastname',
            PARAM_TEXT,
        );
        $mform->setType(
            'config_' . $prefix . '_firstname',
            PARAM_TEXT,
        );
        $mform->setType(
            'config_' . $prefix . '_email',
            PARAM_TEXT,
        );
    }

    /**
     * Method get_data
     *
     * Gets the form data and updates the tags to the course if the data submit succeeds.
     *
     * @return mixed The form data or null if the data submit fails.
     */
    public function get_data() {
        if ($data = parent::get_data()) {
            // If data submit succed we also update tag to course.
            $this->submittags($data);
            // Then we create label activities.
            $config = new stdClass();
            foreach ($data as $field => $value) {
                $configfield = str_replace(
                    'config_',
                    '',
                    $field,
                );
                $config->$configfield = $value;
            }

            $config = (object) array_merge(
                (array) $this->block->config,
                (array) $config,
            );

            $config = createlabelmod($config);

            // Update block config with label ids.
            $data->config_moduleinfotopid = $config->moduleinfotopid;
            $data->config_moduleinfobottomid = $config->moduleinfobottomid;
            // Use instance_config_save to save the configuration changes.
            $this->block->instance_config_save($data);
        }
        return $data;
    }

    /**
     * Method submittags
     *
     * @param stdClass $data The data object containing the course tags.
     *
     * @return void
     */
    private function submittags($data): void {
        global $COURSE;

        // Updating or setting the course tags.
        core_tag_tag::set_item_tags(
            'core',
            'course',
            $COURSE->id,
            context_course::instance($COURSE->id),
            $data->config_infoscourstags,
        );
    }

    /**
     * Indicates whether to display the form when adding.
     *
     * @return bool True if the form should be displayed when adding, false otherwise.
     */
    public static function display_form_when_adding(): bool {
        return false;
    }

    /**
     * Validates data and returns any errors found.
     *
     * @param array $data The data to be validated.
     * @param array $files The files to be validated.
     * @return array An array of errors, if any.
     */
    public function validation($data, $files): array {
        $errors = parent::validation(
            $data,
            $files,
        );

        $requiredFields = [
            'config_license',
            'config_main_lastname',
            'config_main_firstname',
            'config_main_email'
        ];

        foreach ($requiredFields as $field) {
            // Check if the value for the field in $data is empty
            if (empty($data[$field])) {
                // If the field is part of the 'main' group, add the error under 'main_group'
                if (strpos(
                        $field,
                        'config_main_',
                    ) !== false) {
                    $errors['main_group'] = get_string(
                        'requiredauthor',
                        'block_infoscours',
                    );
                } else {
                    $errors[$field] = get_string('required');
                }
            }
        }

        // Validate email fields.
        if (!filter_var(
            $data['config_main_email'],
            FILTER_VALIDATE_EMAIL,
        )) {
            $errors['main_group'] = get_string('invalidemail');
        }

        for ($i = 1; $i <= 3; $i++) {
            $additionalemail = $data["config_additional{$i}_email"];
            // Skipping empty/not provided emails.
            if (!empty($additionalemail) && !filter_var(
                    $additionalemail,
                    FILTER_VALIDATE_EMAIL,
                )) {
                $errors["additional{$i}_group"] = get_string('invalidemail');
            }
        }

        if (!empty($data['config_contact_email']) && !filter_var(
                $data['config_contact_email'],
                FILTER_VALIDATE_EMAIL,
            )) {
            $errors['config_contact_email'] = get_string('invalidemail');
        }

        // Validate integer fields.
        $integerfields = [ 'config_credits_ects' ];
        foreach ($integerfields as $field) {
            if (!empty($data[$field]) && !filter_var(
                    $data[$field],
                    FILTER_VALIDATE_INT,
                )) {
                $errors[$field] = get_string(
                    'invalidintegernumber',
                    'block_infoscours',
                );
            }
        }

        return $errors;
    }
}
