<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Lib files for plugins
 *
 *
 * @package    block_infoscours
 * @copyright  UNESS | Pimenko 2024
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 *
 */

/**
 * Gets the line items that match the current category ID from the provided categories configuration.
 *
 * @param string $configlines The configuration string that contains the categories and line items.
 * @param int $linesids The ID of the current category.
 *
 * @return array An array of line items that match the current category ID, or an empty array if none are found.
 */
function getmatchinglineitems(string $configlines, int $linesids, bool $iscategorieline = false): array {
    // Assume get_parent_category_id() gets the parent category id of a given category
    // This will be used to retrieve the id of the parent category in each iteration if no match is found
    while (true) {
        if (!empty($configlines)) {
            $lines = explode(
                PHP_EOL,
                $configlines,
            );
            if (!empty($lines)) {
                foreach ($lines as $line) {
                    $lineitems = explode(
                        '|',
                        $line,
                    );
                    if (!empty($lineitems)) {
                        if ($lineitems[0] == $linesids) {
                            return $lineitems;
                        }
                    }
                }
            }
        }
        if ($iscategorieline) {
            $category = core_course_category::get(
                $linesids,
                MUST_EXIST,
                true,
            );
            $linesids = $category->get_parent_coursecat()->id;
            if ($linesids == null) {
                break;
            }
        } else {
            break;
        }
    }
    return [];
}

/**
 * Given an object containing all the necessary data,
 * (defined by the form in mod_form.php) this function
 * will create a new instance and return the id number
 * of the new instance.
 *
 * @param object $label
 * @return bool|int
 * @deprecated This will no longer be needed after Moodle 4.3 please delete this later
 * and use label_add_instance instead of labeladdforoldmoodle
 * @global $DB
 */
function labeladdforoldmoodle($label) {
    global $DB;

    if (!$label->name) {
        $label->name = get_label_name($label);
    }
    $label->timemodified = time();

    $id = $DB->insert_record(
        "label",
        $label,
    );

    $completiontimeexpected = !empty($label->completionexpected) ? $label->completionexpected : null;
    \core_completion\api::update_completion_date_event(
        $label->coursemodule,
        'label',
        $id,
        $completiontimeexpected,
    );

    return $id;
}

/**
 * Creates a label module and adds it to the course based on the provided configuration.
 *
 * @param stdClass $blockconfig The configuration object for the label module.
 *
 * @return stdClass The updated configuration object with the module ID added.
 */
function createlabelmod($blockconfig) {
    global $COURSE, $DB, $OUTPUT, $CFG, $PAGE;

    // Check if a module id exists in the block config.
    if (!empty($blockconfig->moduleinfotopid)) {
        // If it exists, delete this module.
        course_delete_module($blockconfig->moduleinfotopid);
        $blockconfig->moduleinfotopid = null;
    }

    // Check if a module id exists in the block config.
    if (!empty($blockconfig->moduleinfobottomid)) {
        // If it exists, delete this module.
        course_delete_module($blockconfig->moduleinfobottomid);
        $blockconfig->moduleinfobottomid = null;
    }

    $topmoduleinfo = new stdClass();
    $topmoduleinfo->modulename = 'label';
    // This is the module type.
    $topmoduleinfo->name = 'Label top';
    // This is the name of the label.
    if (is_array($blockconfig)) {
        $blockconfig = (object) $blockconfig;
    }
    $templatecontext = clone $blockconfig;

    $categoriesconfig = $CFG->block_infoscours_categories;
    $currentcategoryid = $PAGE->course->category;
    $categmatchingitems = getmatchinglineitems(
        $categoriesconfig,
        $currentcategoryid,
        true,
    );

    $schoolfields = [
        'main_school',
        //'additional1_school',
        //'additional2_school',
        //'additional3_school'
    ];

    foreach ($schoolfields as $field) {
        if (isset($templatecontext->$field)) {
            $result = getmatchinglineitems(
                $CFG->block_infoscours_schoollist,
                $templatecontext->$field,
                false,
            );
            if (!$result) {
                continue;
            }
            $formattedResult = [
                0 => $result[0],
                'schoolname' => $result[1],
                'schoollogo' => rtrim($result[2])
            ];
            $templatecontext->schools[] = $formattedResult;
        }
    }

    if (isset($templatecontext->schools)) {
        $templatecontext->displayschools = true;
    }

    // Clean and filter the text values.
    $templatecontext->main_lastname = format_string($templatecontext->main_lastname);
    $templatecontext->main_firstname = format_string($templatecontext->main_firstname);

    $groups = [
        'additional1' => [
            'lastname',
            'firstname',
            'fonction'
        ],
        'additional2' => [
            'lastname',
            'firstname',
            'fonction'
        ],
        'additional3' => [
            'lastname',
            'firstname',
            'fonction'
        ],
    ];

    foreach ($groups as $group => $attributes) {

        // Init values
        $lastname = '';
        $firstname = '';
        $fonction = '';

        foreach ($attributes as $attribute) {
            $fullAttribute = "{$group}_{$attribute}";

            if ($attribute === 'lastname') {
                $lastname = isset($templatecontext->$fullAttribute) ? format_string($templatecontext->$fullAttribute) : '';
            } else if ($attribute === 'firstname') {
                $firstname = isset($templatecontext->$fullAttribute) ? format_string($templatecontext->$fullAttribute) : '';
            } else if ($attribute === 'fonction') {
                $fonction = isset($templatecontext->$fullAttribute) ? format_string($templatecontext->$fullAttribute) : '';
            }
        }

        // Check if lastname and firstname are not empty
        if ($lastname !== '' && $firstname !== '' && $fonction !== '') {
            $templatecontext->$group = "$lastname $firstname - $fonction";
        } else if ($lastname !== '' && $firstname !== '') {
            $templatecontext->$group = "$lastname $firstname";
        }
    }

    $templatecontext->bibliography = isset($templatecontext->bibliography['text']) ? format_text(
        $templatecontext->bibliography['text'],
        $templatecontext->bibliography['format'],
    ) : '';
    $templatecontext->presentation_cours = isset($templatecontext->presentation_cours['text']) ? format_text(
        $templatecontext->presentation_cours['text'],
        $templatecontext->presentation_cours['format'],
    ) : '';
    $templatecontext->prerequis = isset($templatecontext->prerequis['text']) ? format_text(
        $templatecontext->prerequis['text'],
        $templatecontext->prerequis['format'],
    ) : '';
    $templatecontext->objectives = isset($templatecontext->objectives['text']) ? format_text(
        $templatecontext->objectives['text'],
        $templatecontext->objectives['format'],
    ) : '';
    $templatecontext->modes_evaluation = isset($templatecontext->modes_evaluation['text']) ? format_text(
        $templatecontext->modes_evaluation['text'],
        $templatecontext->modes_evaluation['format'],
    ) : '';
    $templatecontext->cycle = !empty($templatecontext->cycle) ? getcyclesoptions($templatecontext->cycle) : 0;
    $templatecontext->discipline = !empty($templatecontext->discipline) ? getdisciplinesoptions($templatecontext->discipline) : 0;
    $templatecontext->niveau = !empty($templatecontext->niveau) ? getniveauxoptions($templatecontext->niveau) : 0;

    $templatecontext->urllogo1 = $categmatchingitems[2];
    $templatecontext->alternatetext1 = $categmatchingitems[3];
    $templatecontext->urllogo2 = $categmatchingitems[4];
    $templatecontext->alternatetext2 = $categmatchingitems[5];
    $templatecontext->urllogo3 = $categmatchingitems[6];
    $templatecontext->alternatetext3 = $categmatchingitems[7];
    $templatecontext->urllogo4 = $categmatchingitems[8];
    $templatecontext->alternatetext4 = $categmatchingitems[9];

    $templatecontext->displaycourstags = true;

    // Make sure each string in 'infoscourstags' is cleaned.
    if (isset($templatecontext->infoscourstags) && !empty($templatecontext->infoscourstags)) {
        // Make sure each string in 'infoscourstags' is cleaned.
        $templatecontext->infoscourstags = array_map(
            'format_string',
            $templatecontext->infoscourstags,
        );
    } else {
        $templatecontext->displaycourstags = false;
        // Alternatively, set a default value or handle it appropriately.
        $templatecontext->infoscourstags = [];
    }

    // continue with the existing code

    $templatecontext->logo_license_url = '';
    $templatecontext->license_text_url = '';
    $licenseconfig = $CFG->block_infoscours_licenses;
    $matchinglicense = getmatchinglineitems(
        $licenseconfig,
        $templatecontext->license,
    );
    if ($matchinglicense) {
        $templatecontext->licensename = format_string($matchinglicense[1]);
        $templatecontext->logo_license_url = $matchinglicense[3];
        $templatecontext->license_text_url = $matchinglicense[2];
    }

    $templatecontext->displayadditionnalauthors = false;
    if ($templatecontext->additional1_lastname || $templatecontext->additional1_firstname ||
        $templatecontext->additional2_lastname || $templatecontext->additional2_firstname ||
        $templatecontext->additional3_lastname || $templatecontext->additional3_firstname) {
        $templatecontext->displayadditionnalauthors = true;
    }

    $templatecontext->authorteam = isset($templatecontext->authorteam['text']) ? format_text(
        $templatecontext->authorteam['text'],
        $templatecontext->authorteam['format'],
    ) : '';

    $topmoduleinfo->intro = $OUTPUT->render_from_template(
        'block_infoscours/contentforlabeltop',
        $templatecontext,
    );

    $topmoduleinfo->introformat = FORMAT_HTML;
    // This is format of label's content.
    $topmoduleinfo->visible = true;
    $topmoduleinfo->section = '0';
    // This is the section number in the course where the module will be added.
    $topmoduleinfo->course = $COURSE->id;
    // This is the course id.
    $topmoduleinfo->module = $DB->get_field(
        'modules',
        'id',
        [ 'name' => $topmoduleinfo->modulename ],
    );

    $topmoduleinfo->coursemodule = null;
    $topmoduleinfo->instance = labeladdforoldmoodle($topmoduleinfo);
    $topmoduleinfo->coursemodule = add_course_module($topmoduleinfo);

    $sectionrec = $DB->get_record(
        'course_sections',
        [
            'course' => $COURSE->id,
            'section' => $topmoduleinfo->section,
        ],
        '*',
        MUST_EXIST,
    );
    $sequence = explode(
        ",",
        $sectionrec->sequence,
    );
    $newsequence = $topmoduleinfo->coursemodule;

    foreach ($sequence as $module) {
        if ($module != $newsequence) {
            $newsequence .= ',' . $module;
        }
    }

    $DB->set_field(
        'course_sections',
        'sequence',
        $newsequence,
        [ 'id' => $sectionrec->id ],
    );

    $botmoduleinfo = new stdClass();
    $botmoduleinfo->modulename = 'label';
    // This is the module type.
    $botmoduleinfo->name = 'Bot label';
    // This is the name of the label.

    $templatecontext->urllogo1 = $categmatchingitems[2];
    $templatecontext->alternatetext1 = $categmatchingitems[3];
    $templatecontext->urllogo2 = $categmatchingitems[4];
    $templatecontext->alternatetext2 = $categmatchingitems[5];
    $templatecontext->urllogo3 = $categmatchingitems[6];
    $templatecontext->alternatetext3 = $categmatchingitems[7];
    $templatecontext->urllogo4 = $categmatchingitems[8];
    $templatecontext->alternatetext4 = $categmatchingitems[9];

    $botmoduleinfo->intro = $OUTPUT->render_from_template(
        'block_infoscours/contentforbotlabel',
        $templatecontext,
    );
    $botmoduleinfo->introformat = FORMAT_HTML;
    // This is format of label's content.
    $botmoduleinfo->visible = true;
    $botmoduleinfo->section = '0';
    // This is the section number in the course where the module will be added.
    $botmoduleinfo->course = $COURSE->id;
    // This is the course id.
    $botmoduleinfo->module = $DB->get_field(
        'modules',
        'id',
        [ 'name' => $botmoduleinfo->modulename ],
    );

    $lastsectionwithmodule = $DB->get_records_sql(
        "SELECT * FROM {course_sections} WHERE  course = ? AND sequence !='' ORDER BY section DESC LIMIT 1",
        [ $COURSE->id ],
    );

    $lastsection = reset($lastsectionwithmodule);
    $botmoduleinfo->section = $lastsection->section;

    // Create a new module.
    $botmoduleinfo->instance = labeladdforoldmoodle($botmoduleinfo);
    $botmoduleinfo->coursemodule = add_course_module($botmoduleinfo);

    // Update the sequence of the last section with the new module.
    $lastsection->sequence = $lastsection->sequence . ',' . $botmoduleinfo->coursemodule;
    $DB->update_record(
        'course_sections',
        $lastsection,
    );

    $blockconfig->moduleinfotopid = $topmoduleinfo->coursemodule;
    $blockconfig->moduleinfobottomid = $botmoduleinfo->coursemodule;

    return $blockconfig;
}

/**
 * Retrieves the dropdown options based on the provided options definition and optional ID.
 *
 * @param array $optionsDef An associative array representing the options definition.
 *                          The keys represent the option IDs and the values represent the option labels.
 * @param mixed $id (optional) The ID of the selected option. Defaults to null.
 *                  It is used to retrieve the label for the specific option.
 *
 * @return mixed An array of option labels if $id is null or invalid, or a single option label if $id is valid.
 */
function getdropdownoptions(array $optionsDef, $id = null) {
    $options = array_map(function($optionKey) {
        return get_string(
            $optionKey,
            'block_infoscours',
        );
    },
        $optionsDef);

    // Set a "none" value to reset if needed.
    array_unshift(
        $options,
        get_string('none'),
    );

    return ($id !== null && $id >= 0 && array_key_exists(
            $id,
            $options,
        )) ? $options[$id] : $options;
}

/**
 * Gets the cycle options for a given ID.
 *
 * @param mixed $id (Optional) The ID of the cycle. Default is null.
 *
 * @return array An array of cycle options based on the given ID.
 */
function getcyclesoptions($id = null) {
    return getdropdownoptions(
        [
            'cycle1',
            'cycle2',
            'cycle3',
            'formation_continue',
            'tout_public'
        ],
        $id,
    );
}

/**
 * Gets the options for the disciplines dropdown.
 *
 * @param int|null $id The ID of the selected discipline option (optional).
 *
 * @return array An array of discipline options. If an $id is provided, the selected discipline option will be returned.
 */
function getdisciplinesoptions($id = null) {
    return getdropdownoptions(
        [
            'medecine',
            'odontologie',
            'pharmacie',
            'maieutique',
            'paramedical',
            'pass',
            'las',
            'staps',
            'sante',
            'transversal',
        ],
        $id,
    );
}

/**
 * Gets the dropdown options for the "Niveaux" field.
 *
 * @param mixed $id The optional ID parameter.
 *
 * @return array The dropdown options for the "Niveaux" field.
 */
function getniveauxoptions($id = null) {
    return getdropdownoptions(
        [
            'infra-bac',
            'L1',
            'L2',
            'L3',
            'M1',
            'M2',
            'doctorat',
            'sante_year1',
            'sante_year2',
            'sante_year3',
            'sante_year4',
            'sante_year5',
            'sante_year6',
            'du_diu',
        ],
        $id,
    );
}