<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Tests the `getniveauxoptions()` function to ensure it returns the correct string when given an ID.
 * Tests the `getcyclesoptions()` function to ensure it returns the correct string when given an ID.
 * Tests the `getdisciplinesoptions()` function to ensure it returns the correct string when given an ID.
 *
 * This method generates an expected string key based on the given ID using a match statement. It then retrieves
 * the expected string value using the generated key. The `getniveauxoptions()` function is called with the current
 * ID, and the resulting string is compared to the expected string using the `assertEquals()` method.
 *
 * @return void
 */
class options_test extends advanced_testcase {

    /**
     * Tests the `getcyclesoptions()` function to ensure it returns the correct string when given an ID.
     *
     * This method generates an expected string key based on the given ID using a match statement. It then retrieves
     * the expected string value using the generated key. The `getcyclesoptions()` function is called with the current
     * ID, and the resulting string is compared to the expected string using the `assertEquals()` method.
     *
     * @return void
     */
    public function test_getcyclesoptions_returns_correct_string_when_given_id() {
        // You can test as many IDs as you like. For simplicity,
        // this code assumes you have 5 different cycles.
        for ($cycleid = 0; $cycleid < 5; $cycleid++) {

            // Generate the name of the string key from the cycle ID,
            // using a match based on order.
            $expectedKey = match ($cycleid) {
                0 => 'cycle1',
                1 => 'cycle2',
                2 => 'cycle3',
                3 => 'formation_continue',
                4 => 'tout_public',
            };

            // Retrieve the expected string value using the name of the string key.
            $expectedResult = get_string($expectedKey, 'block_infoscours');

            // Call the getcyclesoptions() function with the current cycle ID.
            $result = getcyclesoptions($cycleid);

            // The output string should match the expected string.
            $this->assertEquals($expectedResult, $result);
        }
    }

    /**
     * Tests the `getcyclesoptions()` function to ensure it returns an array when not given an ID.
     *
     * This method calls the `getcyclesoptions()` function without any argument and assigns the result to the `$result`
     * variable. It then asserts that the result is an array using the `assertIsArray()` method. It also asserts that the
     * resulting array contains 5 cycle options using the `assertCount()` method. Finally, it asserts that all the expected
     * cycle options are present in the resulting array using the `assertContains()` method.
     *
     * @return void
     */
    public function test_getcyclesoptions_returns_array_when_not_given_id() {
        // Call the getcyclesoptions() function without any argument.
        $result = getcyclesoptions();

        // The result should be an array.
        $this->assertIsArray($result);

        // It should return 5 cycle options.
        $this->assertCount(5, $result);

        // Check all the cycle options are present in the resulting array.
        $this->assertContains(get_string('cycle1', 'block_infoscours'), $result);
        $this->assertContains(get_string('cycle2', 'block_infoscours'), $result);
        $this->assertContains(get_string('cycle3', 'block_infoscours'), $result);
        $this->assertContains(get_string('formation_continue', 'block_infoscours'), $result);
        $this->assertContains(get_string('tout_public', 'block_infoscours'), $result);
    }

    /**
     * Test if the getdisciplinesoptions() function returns the correct string when given an ID.
     *
     * This method tests the getdisciplinesoptions() function by checking if the output string
     * matches the expected string for each given ID. It uses a match based on order to generate
     * the name of the string key from the discipline ID. It then retrieves the expected string
     * value using the calculated key. The function is called with the current discipline ID and
     * the output string is compared with the expected string using the assertEquals() method.
     *
     * @return void
     */
    public function test_getdisciplinesoptions_returns_correct_string_when_given_id() {
        // You can test as many IDs as you like. This code tests IDs 0 to 9, as there are 10 options.
        for ($disciplineid = 0; $disciplineid < 10; $disciplineid++) {
            // Generate the name of the string key from the discipline ID, using a match based on order.
            $expectedKey = match ($disciplineid) {
                0 => 'medecine',
                1 => 'odontologie',
                2 => 'pharmacie',
                3 => 'maieutique',
                4 => 'paramedical',
                5 => 'pass',
                6 => 'las',
                7 => 'staps',
                8 => 'sante',
                9 => 'transversal',
            };
            // Retrieve the expected string value using the calculated key above.
            $expectedResult = get_string($expectedKey, 'block_infoscours');

            // Call the getdisciplinesoptions() function with the current discipline ID.
            $result = getdisciplinesoptions($disciplineid);

            // The output string should match the expected string.
            $this->assertEquals($expectedResult, $result);
        }
    }

    /**
     * Test if the getdisciplinesoptions() function returns an array when not given an ID.
     *
     * This method tests the getdisciplinesoptions() function by calling it without any argument
     * and checking if the result is an array. It then asserts that the count of elements in the
     * array is 10, as per the function's logic. Additionally, it checks a few options in the array
     * to ensure they match the expected localized strings.
     *
     * @return void
     */
    public function test_getdisciplinesoptions_returns_array_when_not_given_id() {
        // Call the getdisciplinesoptions() function without any argument.
        $result = getdisciplinesoptions();

        // The result should be an array.
        $this->assertIsArray($result);

        // As per the function's logic, it should return 10 discipline options.
        // Therefore, the count of elements in the array should be 10.
        $this->assertCount(10, $result);

        // You can also check a few options to ensure they match the expected values.
        // Replace 'option_string' with the expected localized string.
        $this->assertContains(get_string('medecine', 'block_infoscours'), $result);
        $this->assertContains(get_string('odontologie', 'block_infoscours'), $result);
        $this->assertContains(get_string('pharmacie', 'block_infoscours'), $result);
        $this->assertContains(get_string('maieutique', 'block_infoscours'), $result);
        $this->assertContains(get_string('paramedical', 'block_infoscours'), $result);
        $this->assertContains(get_string('pass', 'block_infoscours'), $result);
        $this->assertContains(get_string('las', 'block_infoscours'), $result);
        $this->assertContains(get_string('staps', 'block_infoscours'), $result);
        $this->assertContains(get_string('sante', 'block_infoscours'), $result);
        $this->assertContains(get_string('transversal', 'block_infoscours'), $result);
    }

    /**
     * Tests the `getniveauxoptions()` function to ensure it returns the correct string when given an ID.
     *
     * This method generates an expected string key based on the given ID using a match statement. It then retrieves
     * the expected string value using the generated key. The `getniveauxoptions()` function is called with the current
     * ID, and the resulting string is compared to the expected string using the `assertEquals()` method.
     *
     * @return void
     */
    public function test_getniveauxoptions_returns_correct_string_when_given_id() {
        // You can test as many IDs as you like. For simplicity, this code assumes you have 5 different levels.
        for ($levelid = 0; $levelid < 14; $levelid++) {
            // Generate the name of the string key from the level ID, using a match based on order.
            $expectedKey = match ($levelid) {
                0 => 'infra-bac',
                1 => 'L1',
                2 => 'L2',
                3 => 'L3',
                4 => 'M1',
                5 => 'M2',
                6 => 'doctorat',
                7 => 'sante_year1',
                8 => 'sante_year2',
                9 => 'sante_year3',
                10 => 'sante_year4',
                11 => 'sante_year5',
                12 => 'sante_year6',
                13 => 'du_diu',
            };

            // Retrieve the expected string value using the name of the string key.
            $expectedResult = get_string($expectedKey, 'block_infoscours');

            // Call the getniveauxoptions() function with the current level ID.
            $result = getniveauxoptions($levelid);

            // The output string should match the expected string.
            $this->assertEquals($expectedResult, $result);
        }
    }

    /**
     * Tests the `getniveauxoptions()` function to ensure it returns an array when not given an ID.
     *
     * This method calls the `getniveauxoptions()` function without any argument and assigns the result to the `$result`
     * variable. It then asserts that the `$result` is an array using the `assertIsArray()` method. It also asserts that
     * the `$result` array contains 14 level options using the `assertCount()` method.
     *
     * Furthermore, this method checks that all the expected options are present in the `$result` array using multiple
     * `assertContains()` method calls.
     *
     * @return void
     */
    public function test_getniveauxoptions_returns_array_when_not_given_id() {
        // Call the getniveauxoptions() function without any argument.
        $result = getniveauxoptions();

        // The result should be an array.
        $this->assertIsArray($result);

        // It should return 14 level options.
        $this->assertCount(14, $result);

        // Check all the options are present in the resulting array.
        $this->assertContains(get_string('infra-bac', 'block_infoscours'), $result);
        $this->assertContains(get_string('L1', 'block_infoscours'), $result);
        $this->assertContains(get_string('L2', 'block_infoscours'), $result);
        $this->assertContains(get_string('L3', 'block_infoscours'), $result);
        $this->assertContains(get_string('M1', 'block_infoscours'), $result);
        $this->assertContains(get_string('M2', 'block_infoscours'), $result);
        $this->assertContains(get_string('doctorat', 'block_infoscours'), $result);
        $this->assertContains(get_string('sante_year1', 'block_infoscours'), $result);
        $this->assertContains(get_string('sante_year2', 'block_infoscours'), $result);
        $this->assertContains(get_string('sante_year3', 'block_infoscours'), $result);
        $this->assertContains(get_string('sante_year4', 'block_infoscours'), $result);
        $this->assertContains(get_string('sante_year5', 'block_infoscours'), $result);
        $this->assertContains(get_string('sante_year6', 'block_infoscours'), $result);
        $this->assertContains(get_string('du_diu', 'block_infoscours'), $result);
    }
}
